
#!/usr/bin/perl
use integer;
#
#  
#############################################################################  
#                                                                           #
#        Extraction de terminologie et statistiques pour l'anglais          #
#                                                                           #
#############################################################################
#
#
#############################################################################
# Auteur :                                                                  #
#                                                                           #
# Cedric PRAT                                                               #  
#                                                                           #
# Modification : Samuel DUFOUR-KOWALSKI 11/11/2003
#                                                                           #     
# Adaptation format d'entre et de sortie                                   #
# Adaptation fonction morphologie                                           #
# gestion des structure et identificateur                                   #
#############################################################################  


# Utilisation des expressions rgulires dfinies dans le fichier ang_def.pl 
require ("lib/en_def.pl");

# Utilisation des derivations morphologiques dfinies dans le fichier ang_def2.pl 
require ("lib/en_def2.pl");

#exceptions
require ("lib/en_def3.pl");

# Variables globales du programme  
  
       my %Mots;  # Tableau de Hachage stockant tous les mots rencontrs dans le texte (cl : (flexion, tiquette,lemme))
  
       my %Parc2; # Tableau de hachage stockant les patrons recherchs

       my %Deb; # Tableau de hachage stockant les patrons trouves commencant par un lemme donne
                # ce tableau comporte trois cles: le lemme de debut, le lemme de fin et la suite de lemmes
                # il sert a comptabiliser toutes les frequences, la longueur du patron, la suite de lemmes et l'indice y correspondant.
                # ce tableau comporte trois cles: 
                # le lemme de debut : $Deb{$premierLemme}
                # le lemme de fin : $Deb{$premierLemme}{$dernierLemme}
                # la suite de lemmes : $Deb{$premierLemme}{$dernierLemme}{$chaineLemmes}

                # il sert a comptabiliser toutes les frequences, la longueur du patron, 
                # la suite de lemmes et l'indice y correspondant.
                # $Deb{'nb'} : la frequence totale d'occurrences de patrons rencontres
                # $Deb{$premierLemme}{'nb'} : la frequence du premier lemme au sein 
                #                               d'occurrences de patrons rencontres
                # $Deb{$premierLemme}{$dernierLemme}{'nb'} : la frequence du couple de lemme
                # $Deb{$premierLemme}{$dernierLemme}{$chaineLemmes}[0] : la frequence de la suite
                #                                                        de lemmes
                # $Deb{$premierLemme}{'sommeLog'}: calcul de la diversite du premier lemme

                # le type de structure de base  rencontr : 
                # pour le premier lemme : $Deb{$premierLemme}{'struct'}
                # pour le couple : $Deb{$premierLemme}{$dernierLemme}{'struct'}
                # struct ::=  "na", "npn", "npna", "npnn", "npnng", "npnpn" ou "nv"
                
                # l'identifiant unique du couple de lemme
                # $Deb{$premierLemme}{$dernierLemme}{'id'}
                
                # 3/12/03 BD : rajout etiquette du deuxieme lemme pour regroupement
                #      morphologique aprs la structure na :
                # $Deb{$premierLemme}{$dernierLemme}{'struct'}{'etiq'}

       my %Fin; # Tableau de hachage stockant les patrons trouves finissant par un lemme donne
                # ce tableau comporte trois cles: le lemme de fin,  le lemme de debut et la suite de lemmes
                # il ne sert qu'a comptabiliser les frequences
 
       my %AdjRel; # tableau des adjectifs

       # Rcupration du nom du fichier pass un paramtre
       $parametre=$ARGV[0];  

       # Cration du fichier dans lequel sera crit le rsultat de la recherche de patrons
       open(SORTIE, ">REA/temp.txt");

       # Cration du fichier dans lequel sera crit toutes les informations sur les patrons
       open(SORTIE2, ">REA/temp2.txt");

       # Cration du fichier dans lequel seront ecrits tous les adjectifs relationnels
       open(SORTIE3, ">REA/temp3.txt");

       # Cration du fichier dans lequel seront ecrits tous les patrons comprenant des adjectifs relationnels
       open(SORTIE4, ">REA/relationnel.txt");



#####################################################################################################
#####################################################################################################
#                                          FONCTIONS                                                #
#####################################################################################################
#####################################################################################################

#####################################################################################################
# Fonction d'analyse de l'entre                                                                    #
# supprime les balises et enregistre les info correspondates
#####################################################################################################
# Paramtres :                                                                                      # 
#  $_[0] : ligne  analyser                                                                         #
#####################################################################################################

$baliseAB=0;
sub AnalyseEntree
{
    my $ligne=shift @_;
    chomp($ligne); 

    #BALISE AB
    if($ligne =~ /<AN>/)
    { $baliseAB=1; }


    # recupre le n de texte format 1
    if($baliseAB==1 && $ligne =~ /($DATE) ($SEP) ($REF)/) 
    { 
     	$annee = recupAnnee($1); 
     	$numTexte = recupNumText( $3 ); 

	#supprime la ligne 
	$ligne="";
    } 

    if($ligne =~ /<\/AN>/)
    { $baliseAB=0; }
    
     # rcupere le n de phrase 
    if($ligne =~ /<ph_nb=[0-9]+>/) 
    { 
	$numPhrase = recupNumPhrase($ligne);
    } 
    
    if($ligne =~ /$TITRE/) 
     { 
	 $titre = 1;
     } 
    
    if($ligne =~ /$TEXTE/) 
    { 
	$titre = 0; 
     } 
    
    # enleve les tags 
    $ligne =~ s/<.+?>//g ; 

    return $ligne;
}

#####################################################################################################
# Fonction renvoyant l'annee                                                                        #
# $_[0] : date liee au texte                                                                        #
#####################################################################################################
sub recupAnnee
{ 
  my $date = $_[0];
  @num = split(/\//,$date); 
		 
  return $num[0]; 
}  

 
#####################################################################################################
# Fonction renvoyant le n de texte                                                                 #
# $_[0] : reference du texte                                                                        #
#####################################################################################################
sub recupNumText 
{ 
  my $ref = $_[0];
  @num = split(/\//,$ref); 
		 
  return $num[0]; 
}  

       
#####################################################################################################
# Fonction renvoyant le n de phrase                                                                #
# $_[0] : phrase entiere                                                                            #
#####################################################################################################
sub recupNumPhrase 
{ 
  my $phrase = $_[0];
  $phrase =~ s/<ph_nb=([0-9]+)>(.+?)/$2/g ; 
	 
  return $1; 
} 




#####################################################################################################
# Fonction qui enregistre les frequences                                                            #
#####################################################################################################
# Paramtres :                                                                                      #   
#  $_[0] : premier lemme                                                                            #
#  $_[1] : dernier lemme                                                                            #
#  $_[2] : chaine avec tous les lemmes                                                              #
#  $_[3] : type de la structure de base ("na", "npn", "npna" ou "nv")                               #
#####################################################################################################

sub EnregistreFreq
{
  my $premierLemme = $_[0];  # premier lemme du patron
  my $dernierLemme = $_[1];  # dernier lemme du patron
  my $chaineLemmes = $_[2];  # chaine contenant le patron  stocker
  my $struct = $_[3];        # structure du patron


  # on comptabilise le nombre total de patrons
  if( exists $Deb{'nb'})
  {
      $Deb{'nb'} += 1;
  }
  else
  {
      $Deb{'nb'} = 1;
  }
  
  # on tient a jour le flag qui dit quelles structures on a deja rencontrees
  # si l'on a deja rencontre ce premier lemme en debut
  if( exists $Deb{$premierLemme}{'struct'})
  { 
    SWITCH: {

      if ($struct eq "an")
      {   $Deb{$premierLemme}{'struct'} = $Deb{$premierLemme}{'struct'} | "1" ; last SWITCH ; };
      if ($struct eq "npn") 
      {   $Deb{$premierLemme}{'struct'} = $Deb{$premierLemme}{'struct'} | "2" ; last SWITCH ; };
      if ($struct eq "nn") 
      {   $Deb{$premierLemme}{'struct'} = $Deb{$premierLemme}{'struct'} | "4" ; last SWITCH ; };
  }#fin SWITCH
  }#finsi premier lemme de dbut
  # si l'on n'a pas encore rencontre ce premier lemme en debut 
  else
  {
      # on initialise le flag
    SWITCH: {

      if ($struct eq "an")
      {   $Deb{$premierLemme}{'struct'} = "1" ; last SWITCH ; };
      if ($struct eq "npn") 
      {   $Deb{$premierLemme}{'struct'} = "2" ; last SWITCH ; };
      if ($struct eq "nn") 
      {   $Deb{$premierLemme}{'struct'} = "4" ; last SWITCH ; };
  }#fin SWITCH
  }


  # si l'on a deja rencontre ce premier lemme en debut
  if( exists $Deb{$premierLemme}{'nb'})
  { 
      $Deb{$premierLemme}{'nb'} += 1;
  }
  # si l'on n'a pas encore rencontre ce premier lemme en debut 
  else
  {
      $Deb{$premierLemme}{'nb'} = 1;
      # on initialise cette valeur utile pour la diversite
      $Deb{$premierLemme}{'sommeLog'} = 0;
  }


  # si l'on a deja rencontre ce dernier lemme en fin
  if( exists $Fin{$dernierLemme}{'nb'})
  { 
      $Fin{$dernierLemme}{'nb'} += 1;
  }
  # si l'on n'a pas encore rencontre ce dernier lemme en fin
  else
  {
      $Fin{$dernierLemme}{'nb'} = 1;
      # on initialise cette valeur utile pour la diversite
      $Fin{$dernierLemme}{'sommeLog'} = 0;
  }


  # on tient a jour le flag qui dit quelles structures on a deja rencontrees
  # si l'on a deja rencontre ce premier lemme en debut
  if( exists $Deb{$premierLemme}{$dernierLemme}{'struct'})
  { 
    SWITCH: {

      if ($struct eq "an")
      {   $Deb{$premierLemme}{$dernierLemme}{'struct'} = $Deb{$premierLemme}{$dernierLemme}{'struct'} | "1" ; last SWITCH ; };
      if ($struct eq "npn") 
      {   $Deb{$premierLemme}{$dernierLemme}{'struct'} = $Deb{$premierLemme}{$dernierLemme}{'struct'} | "2" ; last SWITCH ; };
      if ($struct eq "nn") 
      {   $Deb{$premierLemme}{$dernierLemme}{'struct'} = $Deb{$premierLemme}{$dernierLemme}{'struct'} | "4" ; last SWITCH ; };
  }#fin SWITCH
  }
  # si l'on n'a pas encore rencontre ce premier lemme en debut 
  else
  {

    SWITCH: {

      if ($struct eq "an")
      {   $Deb{$premierLemme}{$dernierLemme}{'struct'} = "1" ; last SWITCH ; };
      if ($struct eq "npn") 
      {   $Deb{$premierLemme}{$dernierLemme}{'struct'} = "2" ; last SWITCH ; };
      if ($struct eq "nn") 
      {   $Deb{$premierLemme}{$dernierLemme}{'struct'} = "4" ; last SWITCH ; };
  }#fin SWITCH
  } 


  if( exists $Deb{$premierLemme}{$dernierLemme}{'nb'})
  { 
      $Deb{$premierLemme}{$dernierLemme}{'nb'} += 1;
      # on actualise les valeurs "nb" et "sommeLog" utiles pour la diversite
      $val = $Deb{$premierLemme}{$dernierLemme}{'nb'};
       
      no integer;
      my $temp = $val*1000*(log $val) - ($val-1)*1000*(log ($val-1));
      use integer;

      $Deb{$premierLemme}{'sommeLog'} += $temp;
  }
  else
  {
      $Deb{$premierLemme}{$dernierLemme}{'nb'} = 1;
      $Deb{$premierLemme}{$dernierLemme}{'id'} = $ident ;
      $ident++;
  }


  if( exists $Fin{$dernierLemme}{$premierLemme}{'nb'})
  { 
      $Fin{$dernierLemme}{$premierLemme}{'nb'} += 1;
     # on actualise les valeurs "nb" et "sommeLog" utiles pour la diversite
      my $val = $Fin{$dernierLemme}{$premierLemme}{'nb'};

      no integer;
      my $temp = $val*1000*(log $val) - ($val-1)*1000*(log ($val-1));
      use integer;

      $Fin{$dernierLemme}{'sommeLog'} += $temp;
  }
  else
  {
      $Fin{$dernierLemme}{$premierLemme}{'nb'} = 1;
      $Fin{$dernierLemme}{$premierLemme}{'id'} = $ident-1 ;
  }


  

  # si l'on a deja rencontre le patron
  if( exists $Deb{$premierLemme}{$dernierLemme}{$chaineLemmes})
  { 
      # on incremente la frequence liee au patron
      $Deb{$premierLemme}{$dernierLemme}{$chaineLemmes}{'nb'} += 1;
  }
  # si l'on n'a pas encore rencontre le patron
  else
  {
      # On initialise la frequence du patron
      $Deb{$premierLemme}{$dernierLemme}{$chaineLemmes}{'nb'} = 1;
  }

}




#####################################################################################################
# Fonction qui simplifie un patrons                                                                 #
#####################################################################################################
# Paramtres :                                                                                      #   
#  $_[0] : type du patron ("BASE", "MODIF", "COORD", "ENUM" ou "ATTR")                              # 
#  $_[1] : type de la structure de base ("na", "npn" ou "npna")                                     #
#####################################################################################################
sub Simplifie
{
    my $type = $_[0];
    my $struct = $_[1];
 
    SWITCH :
    {
          # Si le patron a une structure de type AN et un type ATTR ou a une structure de type NPN
          # on garde comme chaine ledernier et le premier mot
	  if  ((($struct eq "an") and ($type eq "ATTR")) or (($struct eq "npn")))
	  {
	      return -1;
	  }


          # Si le patron a une structure de type AN ou NN
          # on garde comme chaine le premier et le dernier mot
	  if  (($struct eq "an") or ($struct eq "nn"))
	  {
	      return 1;
	  }
    }
}


#####################################################################################################
# Fonction qui reconstruit la suite de flexions                                                     #
#####################################################################################################
# Paramtres :                                                                                      #   
#  $_[0] : tableau contenant les mots du patron                                                     #
#  $_[1] : nombre de mots du patron                                                                 #
#####################################################################################################

sub RecupFlex
{
  # on recupere dans des variables les parametres d'appel de la fonction
  my ($monTab) = $_[0];      # tableau contenant les mots du patron 
  my $l = $_[1];             # nombre de mots du patron

  my $temp = $monTab->[0][0];

  # pour tous les mots du patron
  for($j=1; $j < $l; $j++)
  {
	  # on recupere la suite de flexions
	  $temp = $temp." ".$monTab->[$j][0];
  }

  return $temp;
}


#####################################################################################################
# Fonction qui reconstruit la suite d'etiquettes                                                    #
#####################################################################################################
# Paramtres :                                                                                      #   
#  $_[0] : tableau contenant les mots du patron                                                     #
#  $_[1] : nombre de mots du patron                                                                 #
#####################################################################################################

sub RecupEtiq
{
  # on recupere dans des variables les parametres d'appel de la fonction
  my ($monTab) = $_[0];      # tableau contenant les mots du patron 
  my $l = $_[1];             # nombre de mots du patron

  my $temp = $monTab->[0][1];

  # pour tous les mots du patron
  for($j=1; $j < $l; $j++)
  {
	  # on recupere la suite d'etiquettes
	  $temp = $temp." ".$monTab->[$j][1];
  }

  return $temp;
}

#####################################################################################################
# Fonction qui reconstruit la suite de lemmes                                                       #
#####################################################################################################
# Paramtres :                                                                                      #   
#  $_[0] : tableau contenant les mots du patron                                                     #
#  $_[1] : nombre de mots du patron                                                                 #
#####################################################################################################

sub RecupLem
{
  # on recupere dans des variables les parametres d'appel de la fonction
  my ($monTab) = $_[0];      # tableau contenant les mots du patron 
  my $l = $_[1];             # nombre de mots du patron

  my $temp = $monTab->[0][2];

  # pour tous les mots du patron
  for($j=1; $j < $l; $j++)
  {
	  # on recupere la suite de lemmes
	  $temp = $temp." ".$monTab->[$j][2];
  }

  return $temp;
}

#####################################################################################################
# Fonction qui stocke un patron trouv                                                              #
#####################################################################################################
# Paramtres :                                                                                      #   
#  $_[0] : tableau contenant les mots du patron                                                     #
#  $_[1] : nombre de mots du patron                                                                 #
#  $_[2] : type du patron ("BASE", "MODIF", "COORD", "ENUM" ou "ATTR")                              # 
#  $_[3] : type de la structure de base ("na", "npn", "npna" ou "nv")                               #
#  $_[4] : chaine avec tous les lemmes                                                              #
#  $_[5] : premier lemme                                                                            #
#  $_[6] : dernier lemme                                                                            #
#####################################################################################################

sub RemplitPatron
{
  # on recupere dans des variables les parametres d'appel de la fonction
  my ($monTab) = $_[0]; # tableau contenant les mots du patron 
  my $l = $_[1];        # nombre de mots du patron
  my $type = $_[2];     # type du patron
  my $struct = $_[3];   # structure du patron
  my $chaineLemmes =  $_[4]; # chaine contenant le patron  stocker
  my $premierLemme = $_[5];  # premier lemme du patron
  my $dernierLemme = $_[6];  # dernier lemme du patron

  my $inv = Simplifie($type,$struct);

  if ($inv == -1)
  {
      my $temp = $premierLemme;
      $premierLemme = $dernierLemme;
      $dernierLemme = $temp;
  }

  $chaineLemmes = $premierLemme." ".$dernierLemme;

  # on comptabilise des frequences et on calcule des valeurs utiles a la diversite 
  EnregistreFreq($premierLemme,$dernierLemme,$chaineLemmes,$struct);

  #on recupere la suite de flexions
  my $suiteFlex = RecupFlex($monTab,$l);

  #on recupere la suite d'etiquettes
  my $suiteEtiq = RecupEtiq($monTab,$l);

  #on recupere la suite de lemmes
  my $suiteLem = RecupLem($monTab,$l);

  # on ecrit l'indice d'apparition, les flexions, les etiquettes, la structure du patron, le type du patron, le numero de texte et le numero de phrase
  #print SORTIE2  $premierLemme," ",$dernierLemme," ",$suiteLem," --- ",$suiteEtiq," --- ",$struct," --- ",$type," --- ",$numTexte,"\n";
    print SORTIE2 $premierLemme," ",$dernierLemme," ",$Deb{$premierLemme}{$dernierLemme}{'id'}," --- ",$suiteFlex," --- ",$suiteEtiq," --- ",$suiteLem," --- ",$struct, " --- ",$type," --- ",$annee," --- ",$numTexte," --- ",$numPhrase," --- ",$titre,"\n";
}




#####################################################################################################
# Fonction qui rend compte de l'avancement                                                          #
#####################################################################################################
sub Avancement
{
  # marqueur de temps pour se rendre compte de l'avancement du parcours
  $compte ++;

  # affiche le temps pris tous les 1000 patrons trouves
  if (($compte % 1000) == 1)
  {
        my $newtemps = (time);
        my $modu = ($compte - 1) / 1000;
        print STDOUT $modu," ",$newtemps - $temps,"\n";
        $temps = $newtemps;
  }
}




#####################################################################################################
# Fonction qui elimine certains patrons pour alleger la structure                                   #
#####################################################################################################
sub Elimine
{

    print "ELIMINE-deb\n";
    # pour chaque lemme "lemmedeb" apparaissant en debut de patron
    for $lemmedeb( keys %Deb )
    {
	next if ($lemmedeb eq 'nb');

	# pour chaque lemme "lemmefin"  apparaissant en fin d'un de ces patrons
	for  $lemmefin ( keys %{ $Deb{$lemmedeb} } )
	{
	    next if ($lemmefin eq 'nb');
	    next if ($lemmefin eq 'sommeLog');

	    # pour chaque patron commencant par "lemmedeb" et finissant par "lemmefin" 
	    for $lemmePatron ( keys %{ $Deb{$lemmedeb}{$lemmefin} } )
	    {
		next if ($lemmePatron eq 'nb');
		next if ($lemmePatron eq 'id');
		next if ($lemmePatron eq 'struct');
		
		# on recupere la frequence du patron
		my $freq = $Deb{$lemmedeb}{$lemmefin}{$lemmePatron}{'nb'};

		if ($freq <= 1)
		{
		    delete $Deb{$lemmedeb}{$lemmefin}{$lemmePatron};
		    $Deb{$lemmedeb}{$lemmefin}{'nb'} -=1;
		    $Deb{$lemmedeb}{'nb'} -=1;
		    $Deb{'nb'} -=1;
		}    
		
		my $freq = $Deb{$lemmedeb}{$lemmefin}{'nb'};
		if ($freq <= 1)
		{
		    delete $Deb{$lemmedeb}{$lemmefin};
		    $Deb{$lemmedeb}{'nb'} -=$freq;
		    $Deb{'nb'} -=$freq;
		}    

	    }
	}
	my $freq = $Deb{$lemmedeb}{'nb'};
	if ($freq <= 1)
	{
	    delete $Deb{$lemmedeb};
	    $Deb{'nb'} -=$freq;
	}    
    } 
    print "ELIMINE-fin",$Deb{'nb'},"\n";
}



#####################################################################################################
# Fonction qui stocke un patron trouv                                                              #
#####################################################################################################
# Paramtres :                                                                                      #
#  $_[0] : chaine contenant le patron  stocker                                                     #
#  $_[1] : nombre de mots du patron                                                                 #
#  $_[2] : type de la structure de base ("na", "npn", "npna" ou "nv")                               #
#  $_[3] : type de patron reconnu ("COORD", "MODIF", "BASE", "ENUM", "COORDTETE", "ATTR")           #
#####################################################################################################
sub PremierDernier
{
  # on recupere dans des variables les parametres d'appel
  my $chainePatron = $_[0]; # chaine contenant le patron  stocker
  my $long = $_[1];    # nombre de mots du patron
  my $base = $_[2];    # structure de base ("na", "npn","nn")
  my $patron = $_[3];  # type de patron reconnu (COORD, MODIF, BASE, ENUM, ATTR)

  # on rend compte de l'avancement
  Avancement();

  # elimine certains patrons pour alleger la structure
  if  (($compte % 20000 == 0) and ($compte > 50000))
  {
      #Elimine();
  }

  my @chaines=split(/ /,$chainePatron); # stockage de tous les mots du patron dans un tableau 
	
  my @mot=(); # tableau  2 dimensions [nombre de mots du patron][3] qui stocke les flexion, tiquette et lemme de chaque mot du patron

  # pour chaque mot de l'expression on spare flexion, tiquette et lemme
  for($j=0; $j < $#chaines+1 ; $j++)
  {
    # cas particuler : un signe de ponctuation (virgule...)
    if($chaines[$j] =~ /$VIRG/)
    {
     $mot[$j][0]= ", ";     #flexion
     $mot[$j][1]= "CHRS"; #etiquette
     $mot[$j][2]= ", ";     #lemme
    }
    # cas gnral : un mot
    else
    {
      my @temp=split(/\//,$chaines[$j]);

      $mot[$j][0]= $temp[0]; #flexion

      $mot[$j][1]= $temp[1]; #etiquette

      $mot[$j][2] = lc $temp[2]; #lemme
    }

  }


 
  $motdeb = $mot[0][2];   

  # on reconstruit la chaine de tous les lemmes du patron qui va servir de cle pour %Deb 
  my $chaineLemmes = $mot[0][2];       
  for ($i=1; $i<$#chaines+1; $i++)
  {
      $chaineLemmes = $chaineLemmes." ".$mot[$i][2];
  }

  # on appelle la fonction qui enregistre toutes les donnees relatives au patron
  RemplitPatron(\@mot,$long,$patron,$base,$chaineLemmes,$motdeb,$mot[$#mot][2]);

} # fin de la fonction PremierDernier




#####################################################################################################
# Fonction calculant le "LogLike" pour un couple i/j                                                #
#  $_[0] : nombre de couples i/j                                                                    #
#  $_[1] : nombre de couples commencant par i mais ne finisant pas par j                            #
#  $_[2] : nombre de couples ne commencant pas par i mais finisant par j                            #
#  $_[3] : nombre de couples ne commencant pas par i et ne finisant pas par j                       #
#  $_[4] : nombre de couples au total                                                               #
#####################################################################################################
sub like 
{ 
    my $a = $_[0];
    my $b = $_[1];
    my $c = $_[2];
    my $d = $_[3];
    my $N = $_[4];

    my $res = 0;

    no integer;

    $res = $res + $a*(log $a) + $N*(log $N) - ($a+$b)*(log ($a+$b)) - ($a+$c)*(log ($a+$c));
    if ($b != 0) {$res += $b*(log $b)}; 		 
    if ($c != 0) {$res += $c*(log $c)};
    if ($d != 0) {$res += $d*(log $d)}; 		 
    if (($d != 0) or ($b != 0)) {$res -= ($b+$d)*(log ($b+$d))}; 		 
    if (($d != 0) or ($c != 0)) {$res -= ($c+$d)*(log ($c+$d))}; 		 

    # passage pour ne garder que les 3 premieres decimales et pour rajouter 100 afin de bien trier le fichier par la suite
    $res = ((int ($res * 1000)) / 1000) + 1000;

    use integer;

    return $res; 

 
}  
      
       


#####################################################################################################
# Fonction traitant les bases de structure "na", "npn, "npna"" ou "nv"                              #
#####################################################################################################
# Paramtres :                                                                                      #
#  $_[0] : patron  traiter (base de structure "na","npn", "npna" ou "nv")                          # 
#  $_[1] : structure de la base ("na","npn", "npna" ou "nv")                                        #
#                                                                                                   #
# Retourne le nombre de mots que contient le patron   ($nb)                                         #
#                                                                                                   #
#####################################################################################################


sub traiteBase
{
  my $chaine = $_[0]; # rcupration du patron  traiter
  my $struct = $_[1]; # rcupration de la structure ("na","npn", "npna" ou "nv")

  my $type="BASE";
  
  my @tableau = split(/ /,$chaine); # stockage de tous les mots du patron dans un tableau
  my $nb = $#tableau+1; # rcupration du nombre de mots que contient le patron

  PremierDernier($chaine,$nb, $struct, $type); # enregistrement du patron trouv

  return $nb; # Retourne le nombre de mots que contient le patron
}



#####################################################################################################
# Fonction traitant les modifications de structure "an"                                             #
#####################################################################################################
# Paramtres :                                                                                      #
#  $_[0] : patron  traiter (modification de structure "an")                                        #  
#                                                                                                   #
# Retourne le nombre de mots que contient le patron   ($nb)                                         #
#                                                                                                   #
#####################################################################################################

sub traiteModifAN
{
  my $chaine = $_[0]; # rcupration du patron  traiter

  my $type="MODIF";
  my $struct="an";

  my @tableau = split(/ /,$chaine); # stockage de tous les mots du patron dans un tableau
  my $nb = $#tableau+1; # rcupration du nombre de mots que contient le patron

  PremierDernier($chaine, $nb, $struct, $type); # enregistrement du patron trouv
	     


  return $nb; # Retourne le nombre de mots que contient le patron
}


#####################################################################################################
# Fonction traitant les modifications de structure "npn"                                            #
#####################################################################################################
# Paramtres :                                                                                      #
#  $_[0] : patron  traiter (modification de structure "npn")                                       #  
#                                                                                                   #
# Retourne le nombre de mots que contient le patron   ($nb)                                         #
#                                                                                                   #
#####################################################################################################

sub traiteModifNPN
{
  my $chaine = $_[0]; # rcupration du patron  traiter
  my $type="MODIF";
  my $struct="npn";

  my @tableau = split(/ /,$chaine); # stockage de tous les mots du patron dans un tableau
  my $nb = $#tableau+1; # rcupration du nombre de mots que contient le patron

  PremierDernier($chaine, $nb, $struct, $type); # enregistrement du patron trouv

  return $nb; # Retourne le nombre de mots que contient le patron
}




#####################################################################################################
#####################################################################################################
#                                          PROGRAMME PRINCIPAL                                      #
#####################################################################################################
###################################################################################################**

$ident = 0;
$temps = (time);
$compteLem = 0; 
$compteFlex = 0; 
$compte = 0;

open(FIC2, $parametre) || die("fichier non trouve $!"); #ouverture du fichier en lecture  
  
print "\nParcours du texte : recherche de patrons\n";


       foreach (<FIC2>) 
       { 

	   #analyse l'entree
	   $pointeur=AnalyseEntree($_);
	   
	   next if( $pointeur =~ m/^$/); 
 
	 	
	 # tant que la fin de la ligne n'est pas atteinte, la recherche continue
	 while($pointeur !~ /^$/)
	 {

	   $machaine=" ".$pointeur;


	   SWITCH: {

           # ELIMINATION DES CAS OU L'ON NE TOMBE PAS SUR UN NOM OU UN ADJECTIF : le pointeur passe directement au mot suivant
           if(($machaine !~ /^ ($NOM)(.*)$/) and ($machaine !~ /^ ($ADJ)(.*)$/))   
	   {
	       @tab = split(/ /,$pointeur,2);
	       $pointeur = $tab[1];

	       last SWITCH;
           }


	   # RECHERCHE D'ENUMERATION DE TYPE "an"

	   if($machaine =~ /^ ($ENUMAN)(.*)$/)   
	   {
	     $c=$1;    	    # rcupration du patron trouv dans $c
	     $type="ENUM"; 
	     $struct="an";
	     @tableau1 = split(/ /,$c); # stockage de tous les mots du patron dans un tableau
	     $t = $#tableau1+1; # on rcupre le nombre de mots que contient le patron

	     PremierDernier($c, $t, $struct, $type); # enregistrement du patron trouv

	     # Recherche de l'indice du 1er COO de la chaine 
	     $i = 0;
	     while(not(($tableau1[$i]." ") =~ /$COO/) and ($i < $#tableau1)) 
	     {
	       $i = $i+1;
	       $taille = $i; 
	     }

	     # Le pointeur va se positionner sur le mot suivant le 1er COO
	     @tab = split(/ /,$pointeur,$taille+2);

	     $pointeur = $tab[$taille+1];

	     last SWITCH;
	   }





	   # RECHERCHE DE COORDINATION DE TETE DE TYPE "an"

	   if($machaine =~ /^ ($COORDAN)(.*)$/)   
	   {
	     $c=$1;    	    # rcupration du patron trouv dans $c
	     $type="COORD";
	     $struct="an";
	     @tableau1 = split(/ /,$c); # stockage de tous les mots du patron dans un tableau
	     $t = $#tableau1+1; # on rcupre le nombre de mots que contient le patron

	     PremierDernier($c, $t, $struct, $type); # enregistrement du patron trouv

	     # Recherche de la position du COO
	     $i = 0;
	     while(not(($tableau1[$i]." ") =~ /$COO/) and ($i < $#tableau1)) 
	     {
	       $i = $i+1;
	       $taille = $i;
	     }

	     # Le pointeur va se positionner sur le mot suivant le 1er COO
	     @tab = split(/ /,$pointeur,$taille+2);

	     $pointeur = $tab[$taille+1];

	   last SWITCH;
	   }


	   # RECHERCHE DE COORDINATION DE TETE DE TYPE "npn"

	   if($machaine =~ /^ ($COORDNPN1)(.*)$/)
	   {
	     $c=$1;    	    # rcupration du patron trouv dans $c
	     $type="COORD";
	     $struct="npn";
	     @tableau1 = split(/ /,$c); # stockage de tous les mots du patron dans un tableau
	     $t = $#tableau1+1; # on rcupre le nombre de mots que contient le patron

	     PremierDernier($c, $t, $struct, $type); # enregistrement du patron trouv

	     # Recherche de l'indice du 1er COO de la chaine 
	     $i = 0;
	     while(not(($tableau1[$i]." ") =~ /$COO/) and ($i < $#tableau1)) 
	     {
	       $i = $i+1;
	       $taille = $i; 
	     }

	     # Le pointeur va se positionner sur le mot suivant le 1er COO
	     @tab = split(/ /,$pointeur,$taille+2);

	     $pointeur = $tab[$taille+1];

	     last SWITCH;
	   }


	   # RECHERCHE DE COORDINATION DE TYPE "npn"

	   if($machaine =~ /^ ($COORDNPN2)(.*)$/)   
	   {
	     $c=$1;    	    # rcupration du patron trouv dans $c
	     $type="COORD";
	     $struct="npn";
	     @tableau1 = split(/ /,$c); # stockage de tous les mots du patron dans un tableau
	     $t = $#tableau1+1; # on rcupre le nombre de mots que contient le patron

	     PremierDernier($c, $t, $struct, $type); # enregistrement du patron trouv

	     # Recherche de l'indice du 1er OF de la chaine 
	     $i = 0;
	     while(not(($tableau1[$i]." ") =~ /$PREP_OF/) and ($i < $#tableau1)) 
	     {
	       $i = $i+1;
	       $taille = $i; 
	     }

	     # Le pointeur va se positionner sur le mot suivant le 1er PREP_OF
	     @tab = split(/ /,$pointeur,$taille+2);

	     $pointeur = $tab[$taille+1];

	     last SWITCH;
	   } 




	   # RECHERCHE D'ATTIBUT DE TYPE "an"

	   if ($machaine =~ /^ ($ATTNA)(.*)$/)
	   {
    	     $c=$1;    	    # rcupration du patron trouv dans $c
	     $type="ATTR";
	     $struct="an";
	     @tableau1 = split(/ /,$c); # stockage de tous les mots du patron dans un tableau
	     $t = $#tableau1+1; # on rcupre le nombre de mots que contient le patron

	     PremierDernier($c, $t, $struct, $type); # enregistrement du patron trouv

             # Le pointeur va se positionner sur le dernier mot du patron:
	     @tab = split(/ /,$pointeur,3);
	     $pointeur = $tab[2];
	   
	     last SWITCH;
	   }





	   # RECHERCHE DE MODIFICATION DE TYPE "an"

	   if ($machaine =~ /^ ($MODIFAN)(.*)$/)
	   {
	     $t = traiteModifAN($1);

	     # Le pointeur va se positionner sur le deuxieme mot du le patron
	     @tab = split(/ /,$pointeur,2);
	     $pointeur = $tab[1];
	   
	     last SWITCH;
	   }





	   # RECHERCHE DE MODIFICATION DE TYPE "npn"

	   if ($machaine =~ /^ ($MODIFNPN)(.*)$/)
	   {
  	     $t = traiteModifNPN($1);

 	     # Le pointeur va se positionner sur le troisieme mot du patron trouv
 	     @tab = split(/ /,$pointeur,3);
 	     $pointeur = $tab[2];

	     last SWITCH;
	   }




	   # RECHERCHE DE BASE DE TYPE "an"

	   if($machaine =~ /^ ($BASEAN)(.*)$/)
	   {
	     $t = traiteBase($1,"an");

	     # Le pointeur va se positionner sur le deuxieme mot du patron
	     @tab = split(/ /,$pointeur,2);
	     $pointeur = $tab[1];

	     last SWITCH;
	   } # fin if





	   # RECHERCHE DE BASE DE TYPE "nn"

	   if($machaine =~ /^ ($BASENN)(.*)$/)
	   {
	     $t = traiteBase($1,"nn");

	     # Le pointeur va se positionner sur le deuxieme mot du patron	     
	     @tab = split(/ /,$pointeur,2);
	     $pointeur = $tab[1];

	     last SWITCH;
	   } # fin if





	   # RECHERCHE DE BASE DE TYPE "npn"

	   if($machaine =~ /^ ($BASENPN)(.*)$/)
	   {
	     $t = traiteBase($1,"npn");

	     # Le pointeur va se positionner sur le dernier mot du patron	     
	     @tab = split(/ /,$pointeur,3);
	     $pointeur = $tab[2];

	     last SWITCH;
	   } # fin if




	   # PAS DE BASE NI DE MODIFICATION NI DE COORDONNATION NI D'ENUMERATION TROUVEES : le pointeur passe au mot suivant

	     @tab = split(/ /,$pointeur,2);
	     $pointeur = $tab[1];

           } # fin du switch

	 } # fin du while

     } # fin du foreach(<FIC2>)  

close(FIC2);


##############################################
# recherche des regroupements morphologiques 
###############################################

print "Ecriture du fichier de sortie\n\n";

# on recupere la frequence totale des patrons
$N = $Deb{'nb'};

# pour chaque lemme "lemmedeb" apparaissant en debut de patron
for $lemmedeb( keys %Deb )
{
    next if ($lemmedeb eq 'nb');
    
    # on recupere la frequence des patrons commencant par ce lemme "lemmedeb"
    $freqDeb = $Deb{$lemmedeb}{'nb'};
    
    # pour chaque lemme "lemmefin"  apparaissant en fin d'un de ces patrons
    for  $lemmefin ( keys %{ $Deb{$lemmedeb} } )
    { 
	next if ($lemmefin eq 'nb');
	next if ($lemmefin eq 'sommeLog');
	next if ($lemmefin eq 'struct');
	
	# on recupere la frequence des patrons commencant par ce lemme "lemmedeb"
	$freqFin = $Fin{$lemmefin}{'nb'};
	
	#regourpement morphologique
	Morphologie($lemmedeb, $lemmefin);
	
	# on initialise la frequence des patrons commencant par "lemmedeb" et finissant par "lemmefin" 
	#$freqDebFin = $Deb{$lemmedeb}{$lemmefin}{'nb'};
		
	#genere temp.txt
	sortie_temp_txt($lemmedeb, $lemmefin, $freqDeb, $freqFin);

    }
} 

#on ecrit la frequence totale
print SORTIE $N,"\n";

#on ecrit les adj relationnel dans temp3
for $adj( keys %AdjRel)
{
    #print SORTIE3 $adj,"\n";
    print SORTIE3 $adj," ",1," ( )\n";

}

print "Fin\n\n";	   

#fermeture des fichiers
close(SORTIE);
close(SORTIE2);
close(SORTIE3);
close(SORTIE4);

#####################################################################################################
# Fonction qui relie un patron equivalent morphologiquement a un patron trouve                      #
#####################################################################################################
# Paramtres :                                                                                      #   
#  $_[0] : premier lemme                                                                            #
#  $_[1] : dernier lemme
#####################################################################################################

sub Morphologie
{

  #Dclaration des variables utilises dans Morphologie
    my $premierLemme ; #Premier lemme du tableau Deb
    my $dernierLemme ; #Dernier lemme du tableau Deb
    my $suffG ; #variable auxiliaire qui contient le suffixe general
    my $suffA ;  #variable auxilaire qui contient le suffixe allomorphique
    my $radical ; #variable auxilaire qui contient le radical
    my $consonne ; #variable qui contient consonne
    my $newMot ; #variable auxilaire qui contient le nouveau mot construit
    my $listregles ; #liste de regles associe  chaque suffixe
    my $i ;   #variable de boucle

    # on recupere dans des variables les parametres d'appel de la fonction
    $premierLemme = $_[0];  # premier lemme du patron
    $dernierLemme = $_[1];  # dernier lemme du patron

    #Verification que l'adjectif n'est pas deja dans le tableau
    #des AdjRel
    
    if ((exists ($AdjRel{$premierLemme})) && (exists $Deb{$AdjRel{$premierLemme}}{$dernierLemme}))
    {
	print SORTIE4 $Deb{$premierLemme}{$dernierLemme}{'id'}, " ", $premierLemme," ",$dernierLemme," --- ", $Deb{$AdjRel{$premierLemme}}{$dernierLemme}{'id'}, " ", $AdjRel{$premierLemme}," ", $dernierLemme ,"\n";
    }
    else
    {	
	# fait appel a %MorphAdjRel, defini dans "fr_def2.pl"
	foreach $suffG (keys %MorphAdjRel)
	{

	    # si le dernier lemme finit par ce suffixe
	    if ($premierLemme =~ /(.*)($suffG)$/)
	    {	      
		$listregles = $MorphAdjRel{$suffG} ;
		for ($i=0 ; $i <= $#$listregles ; $i++)
		{
		    # le suffixe d'adjectif avec variante allomorphique est le premier champ
		    $suffA = $listregles->[$i]->[0];
		    # si le dernier lemme finit par ce suffixe allomorphique
		    if ($premierLemme =~ /(.*)($suffA)$/)
		    {
			$radical = $1 ;
			$newMot = $radical.$listregles->[$i]->[1] ;
			if (exists $Deb{$newMot}{$dernierLemme}  && !exists($EXCEPTION_SUFF{$suffG}{$premierLemme}))
			{
			    $AdjRel{$premierLemme} = $newMot ;
			    
			    #BD : affichage en plus de l'identifiant
			    print SORTIE4 $Deb{$premierLemme}{$dernierLemme}{'id'}, " ", $premierLemme," ",$dernierLemme," --- ", $Deb{$newMot}{$dernierLemme}{'id'}, " ", $newMot, " ",$dernierLemme,"\n";

			}
			
		    } #fin if allomorphe
		} #fin For examen des regles
	    } #fin test famille de suffixes
	} #fin exemen famille de suffixes
    } #fin else
}#fin sous fonction morphologie


#######################################################################################
# Fonction qui genere le fichier temp.txt                                             #
#######################################################################################
# Paramtres :                                                                        #   
#  $_[0] : premier lemme                                                              #
#  $_[1] : dernier lemme                                                              #
#  $_[2] : frequence du premier lemme                                                            #
#  $_[3] : frequence du dernier lemme                                                              #
#######################################################################################
                               
sub sortie_temp_txt
{
    my $freqDebFin;
    my $lemmePatron;
    my $freq;


    # on recupere le 1er lemme du tableau Deb
    my $lemmedeb = $_[0];  # premier lemme 
    my $lemmefin = $_[1];  # dernier lemme 
    my $freqDeb = $_[2]; # frequence premier lemme
    my $freqFin = $_[3]; # frequence dernier lemme


    # on initialise la frequence des patrons commencant par "lemmedeb" et finissant par "lemmefin" 
    $freqDebFin = $Deb{$lemmedeb}{$lemmefin}{'nb'};


    print SORTIE $Deb{$lemmedeb}{$lemmefin}{'id'};
    
    # on affiche la frequence des patrons commencant par "lemmedeb" et finissant par "lemmefin"
    print SORTIE " ", $freqDebFin," ", $freqDeb, " ", $freqFin, " ", $lemmedeb," ",$lemmefin," ", $Deb{$lemmedeb}{$lemmefin}{'struct'}, " ";

    # pour chaque patron commencant par "lemmedeb" et finissant par "lemmefin" 
    for $lemmePatron ( keys %{ $Deb{$lemmedeb}{$lemmefin} } )
    {
	next if ($lemmePatron eq 'nb');
	next if($lemmePatron eq 'struct');
	next if ($lemmePatron eq 'id');

	print SORTIE "* ";
	
	# on affiche le lemme pour chaque mot du patron
	print SORTIE $lemmePatron," ";

    }
    
    print SORTIE "\n";
}
